# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class Labelling:
    def cformat(self, nfirst="", nlast="", **kwargs):
        """Controls the graphical display of alphanumeric character strings for

        APDL Command: /CFORMAT
        parameters, components, assemblies, and tables.

        Parameters
        ----------
        nfirst
            Display the first n characters of the parameter, component,
            assembly, or table name, up to 32. Defaults to 32.

        nlast
            Display the last n characters of the parameter, component,
            assembly, or table name, up to 32. Defaults to 0.

        Notes
        -----
        Use this command to control the length of the character string that is
        shown in the graphics window for a parameter, component, assembly, or
        table name.

        The total number of characters (NFIRST + NLAST +3) cannot exceed 32.

        If NFIRST is greater than zero and NLAST = 0, only the NFIRST
        characters are displayed, followed by an ellipsis.

        If NFIRST = 0 and NLAST is greater than zero, only the NLAST characters
        are displayed, preceded by an ellipsis (...).

        If both NFIRST and NLAST are greater than zero, the name will be shown
        as NFIRST, followed by an ellipsis (...), followed by NLAST, up to a
        maximum of 32 characters.

        For example, if NFIRST = 6 and NLAST = 3, and the character string is
        LENGTHOFSIDEONE, then it will appear in the graphics window as
        LENGTH...ONE.

        If the actual length of the character string is less than the specified
        combination of NFIRST + NLAST +3, then the actual string will be used.

        This command is valid in any processor.
        """
        command = f"/CFORMAT,{nfirst},{nlast}"
        return self.run(command, **kwargs)

    def clabel(self, wn="", key="", **kwargs):
        """Specifies contour labeling.

        APDL Command: /CLABEL

        Parameters
        ----------
        wn
            Window number (or ALL) to which command applies (defaults to 1).

        key
            Labeling key:

            0 or 1
                Label contours with legend or color (default).

            -1
                No contour labeling.

            N
                Same as 1 except show alphabetic legend only on every Nth element.

        Notes
        -----
        Labels contours for identification with alphabetic legend for vector
        displays and color for raster displays.  Number of contours is
        automatically reduced to 9 (or fewer) for clarity.  Use /CONTOUR
        command to increase (24 maximum for alphabetic labeling; no limit for
        color labeling).

        This command is valid in any processor.
        """
        command = f"/CLABEL,{wn},{key}"
        return self.run(command, **kwargs)

    def contour(self, wn="", ncont="", vmin="", vinc="", vmax="", **kwargs):
        """Specifies the uniform contour values on stress displays.

        APDL Command: /CONTOUR

        Parameters
        ----------
        wn
            Window number (or ALL) to which command applies (defaults to 1).

        ncont
            Number of contour values. NCONT defaults to 9 for X11 or WIN32 and
            to 128 for X11c or WIN32C.  The default graphics window display for
            3-D devices is a smooth continuous shading effect that spans the
            maximum of 128 contours available. Use the /DV3D command to create
            defined banding for your contour values (values of 9 and 128 are
            displayed in smooth shading only). The legend, however, will
            display only nine color boxes, which span the full range of colors
            displayed in the graphics window.

        vmin
            Minimum contour value.  If VMIN = AUTO, automatically calculate
            contour values based upon NCONT uniformly spaced values over the
            min-max extreme range.  Or, if VMIN = USER, set contour values to
            those of the last display (useful when last display automatically
            calculated contours).

        vinc
            Value increment (positive) between contour values.  Defaults to
            (VMAX-VMIN)/NCONT.

        vmax
            Maximum contour value.  Ignored if both VMIN and VINC are
            specified.

        Notes
        -----
        See the /CVAL command for alternate specifications.  Values represent
        contour lines in vector mode, and the algebraic maximum of contour
        bands in raster mode.

        Note:: : No matter how many contours (NCONT) are specified by /CONTOUR,
        the actual number of contours that appear on your display depends also
        on the device name, whether the display is directed to the screen or to
        a file, the display mode (vector or raster), and the number of color
        planes.  (All these items are controlled by /SHOW settings.) In any
        case, regardless of whether they are smoothed or banded, only 128
        contours can be displayed. See Creating Geometric Results Displays in
        the Basic Analysis Guide for more information on changing the number of
        contours.

        If the current ANSYS graphics are not displayed as Multi-Plots, then
        the following is true:  If the current device is a 3-D device
        [/SHOW,3D], the model contours in all active windows will be the same,
        even if separate /CONTOUR commands are issued for each active window.
        For efficiency, ANSYS 3-D graphics logic maintains a single data
        structure (segment), which contains precisely one set of contours.  The
        program displays the same segment in all windows.  The view settings of
        each window constitute the only differences in the contour plots in the
        active windows.

        This command is valid in any processor.
        """
        command = f"/CONTOUR,{wn},{ncont},{vmin},{vinc},{vmax}"
        return self.run(command, **kwargs)

    def cval(
        self,
        wn="",
        v1="",
        v2="",
        v3="",
        v4="",
        v5="",
        v6="",
        v7="",
        v8="",
        **kwargs,
    ):
        """Specifies nonuniform contour values on stress displays.

        APDL Command: /CVAL

        Parameters
        ----------
        wn
            Window number (or ALL) to which command applies (defaults to 1).

        v1, v2, v3, . . . , v8
            Up to 8 contour values may be specified (in ascending order).  The
            0.0 value (if any) must not be the last value specified.  If no
            values are specified, all contour specifications are erased and
            contours are automatically calculated.

        Notes
        -----
        This command is similar to the /CONTOUR command. With /CVAL, however,
        you define the upper level of each contour band instead of having the
        contours uniformly distributed over the range. The minimum value
        (including a zero value for the first band) for a contour band cannot
        be specified. If you use both /CONTOUR and /CVAL, the last command
        issued takes precedence.

        This command is valid in any processor.
        """
        command = f"/CVAL,{wn},{v1},{v2},{v3},{v4},{v5},{v6},{v7},{v8}"
        return self.run(command, **kwargs)

    def gformat(self, ftype="", nwidth="", dsignf="", **kwargs):
        """Specifies the format for the graphical display of numbers.

        APDL Command: /GFORMAT

        Parameters
        ----------
        ftype
            FORTRAN format types (G is the default if this field is left
            blank.)

            G
                Gxx.yy.  xx and yy are described below.

            F
                Fxx.yy

            E
                Exx.yy

        nwidth
            Total width (12 maximum) of the field (the xx in Ftype).  Defaults
            to 12.

        dsignf
            Number of digits after the decimal point (yy in F or E format) or
            number of significant digits in G format.  Range is 1 to xx-6 for
            Ftype = G or E; and 0 to xx-3 for Ftype = F.  The default is a
            function of Ftype and NWIDTH.

        Notes
        -----
        Lets you control the format of the graphical display of floating point
        numbers.   Issue /GFORMAT,STAT to display the current settings; issue
        /GFORMAT,DEFA to let ANSYS choose the format for the graphical display
        of floating numbers.

        This command is valid in any processor.
        """
        command = f"/GFORMAT,{ftype},{nwidth},{dsignf}"
        return self.run(command, **kwargs)

    def hbc(self, wn="", key="", **kwargs):
        """Determines how boundary condition symbols are displayed in a display

        APDL Command: /HBC
        window.

        Parameters
        ----------
        wn
            Window reference number. This number can be any window numbered 1
            to 5, or ALL (for all active windows). Defaults to 1

        key
            Key to enable/disable hidden surface boundary condition symbol
            display for 2-D graphics devices and to request improved pressure
            contour display for 2-D and 3-D devices: Key = ON, YES or 1 will
            show your BC symbols on the hidden surfaces and use an improved
            pressure contour display. Key = OFF, NO or 0 (default) will hide
            the symbols .
        """
        command = f"/HBC,{wn},{key}"
        return self.run(command, **kwargs)

    def number(self, nkey="", **kwargs):
        """Specifies whether numbers, colors, or both are used for displays.

        APDL Command: /NUMBER

        Parameters
        ----------
        nkey
            Numbering style:

            * ``0`` : Color (terminal dependent) the numbered items
              and show numbers.

            * ``1`` : Color the numbered items.  Do not show the
              numbers.

            * ``2`` : Show the numbers.  Do not color the items.

            * ``-1`` : Do not color the items or show the numbers. For
              contour plots, the resulting display will vary (see below).

        Notes
        -----
        Specifies whether numbers, colors, or both are used for numbering
        displays [/PNUM] of nodes, elements, keypoints, lines, areas, and
        volumes.

        Shading is also available for terminals configured with more than 4
        color planes [/SHOW].  Color automatically appears for certain items
        and may be manually controlled (off or on) for other items.

        When you suppress color (NKEY = -1) your contour plots will produce
        different results, depending on your graphics equipment. For non-3-D
        devices (X11, Win32, etc.) your contour plot will be white (no color).
        For 3-D devices, such as OpenGL, the resulting display will be in
        color.

        The following items are automatically given discrete colors:  Boundary
        condition symbols [/PBC], curves on graph displays, and distorted
        geometry on postprocessing displays.  Contour lines in postprocessing
        displays are automatically colored based upon a continuous, rather than
        a discrete, spectrum so that red is associated with the highest contour
        value.  On terminals with raster capability [/SHOW], the area between
        contour lines is filled with the color of the higher contour.

        Explicit entity colors or the discrete color mapping may be changed
        with the /COLOR command.

        This command is valid in any processor.
        """
        command = f"/NUMBER,{nkey}"
        return self.run(command, **kwargs)

    def pbc(self, item="", key="", min_="", max_="", abs_="", **kwargs):
        """Shows boundary condition (BC) symbols and values on displays.

        APDL Command: /PBC

        Parameters
        ----------
        item
            Label identifying the item:

            U
                Applied translational constraints (UX, UY, UZ).

            ROT
                Applied rotational constraints (ROTX, ROTY, ROTZ).

            TEMP
                Applied temperatures (TEMP, TBOT, TE2, TE3, . . ., TTOP).

            PRES
                Applied fluid pressures.

            V
                Applied flow velocities (VX, VY, VZ).

            VOLT
                Applied voltages.

            MAG
                Applied scalar magnetic potentials.

            A
                Applied vector magnetic potentials.

            CONC
                Concentration.

            CHRG
                Applied electric charge.

            F or FORC
                Applied structural forces (FX, FY, FZ).

            M or MOME
                Applied structural moments (MX, MY, MZ).

            HEAT
                Applied heat flows (HEAT, HBOT, HE2, HE3, . . ., HTOP).

            FLOW
                Applied fluid flow.

            AMPS
                Applied current flow.

            FLUX
                Applied magnetic flux.

            CSG
                Applied magnetic current segments.

            RATE
                Diffusion flow rate.

            MAST
                Master degrees of freedom.

            CP
                Coupled nodes.

            CE
                Nodes in constraint equations.

            NFOR
                POST1 nodal forces.

            NMOM
                POST1 nodal moments

            RFOR
                POST1 reaction forces.

            RMOM
                POST1 reaction moments (MX, MY, MZ).

            PATH
                Path geometry (undistorted) associated with the PATH command after a PDEF or
                PVECT command has been issued.

            ACEL
                Global acceleration (ACELX, ACELY, ACELZ vector).

            OMEG
                Global angular velocity (OMEGX, OMEGY, OMEGZ vector) and acceleration (DOMEGX,
                DOMEGY, DOMEGZ vector).

            WELD
                Applied spotwelds (ANSYS LS-DYNA).

            ALL
                Represents all appropriate labels.

        key
            Symbol key:

            0
                Do not show symbol.

            1
                Show symbol.

            2
                Plot value next to symbol.

        min\\_
            Minimum value in a range of values plotted on screen.

        max\\_
            Maximum value in a range of values plotted on screen.

        abs\\_
            Absolute number.  If KEY = 2 and ABS = 0, a number falling between
            the MIN and MAX is displayed.  If ABS is not specified, it defaults
            to 0.  If KEY = 2 and ABS = 1, an absolute value falling between
            the MIN and MAX is displayed.   ABS = 1 lets you eliminate the
            display of numbers whose absolute values are less than a desired
            tolerance.  For example, if ABS = 1, MIN = 10 and MAX = 1e8, values
            such as .83646 and -5.59737 are not displayed.

        Notes
        -----
        The ``mapdl.pbc`` command adds degree of freedom constraint, force load, and
        other symbols to displays.

        Symbols are applied to the selected nodes only.  All arrow and
        arrowhead symbols are oriented in the nodal coordinate system and lie
        in two perpendicular planes.  Force arrows are scaled proportional to
        their magnitude. (If KEY =  1, use /VSCALE to change arrow length.)
        For scalar quantities, the specific component direction (i.e., x, y, or
        z) of the symbol has no meaning, but the positive or negative sense
        (e.g., positive or negative x) represents a positive or negative scalar
        value, respectively.

        The effects of the ``mapdl.pbc`` command are not cumulative (that is, the
        command does not modify an existing setting from a previously issued
        ``mapdl.pbc`` command).  If you issue multiple ``mapdl.pbc`` commands
        during an analysis, only the setting specified by the most recent
        ``mapdl.pbc`` command applies.

        Use ``mapdl.pstatus()`` or ``mapdl.pbc('STAT')`` to display settings.
        Use ``mapdl.pbc('DEFA')`` to reset all specifications back to default.
        See the ``mapdl.psf`` and ``mapdl.pbf`` commands for other display symbols.

        In a cyclic symmetry analysis, the ``mapdl.pbc`` command is deactivated when
        cyclic expansion is active (/CYCEXPAND,,ON). To view boundary
        conditions on the basic sector, deactivate cyclic expansion
        (/CYCEXPAND,,OFF) and issue this command: ``mapdl.pbc('ALL', 1)``

        Issuing the command ``mapdl.pbc('PATH', 1)`` displays all defined paths.

        The ``mapdl.pbc`` command is valid in any processor.

        .. note:
            In APDL the /PBC command has an unused 3rd argument. This has been
            removed in PyMAPDL because of its redundancy, and should be
            kept in mind when translating between the two.

        Examples
        --------
        On all subsequent plots (after this command). Activate display of
        translational boundary condition symbols.

        >>> mapdl.pbc('U', 1)

        Display enforced nodal temperatures

        >>> mapdl.pbc('TEMP', 1)
        """
        command = f"/PBC,{item},,{key},{min_},{max_},{abs_}"
        return self.run(command, **kwargs)

    def pbf(self, item="", key="", **kwargs):
        """Shows magnitude of body force loads on displays.

        APDL Command: /PBF

        Parameters
        ----------
        item
            Label identifying the item:

            TEMP
                Applied temperatures.

            FLUE
                Applied fluences.

            HGEN
                Applied heat generation rates.

            JS
                Applied current density magnitude.

            JSX
                X-component of current density.

            JSY
                Y-component of current density.

            JSZ
                Z-component of current density.

            PHASE
                Phase angle of applied load.

            MVDI
                Applied magnetic virtual displacements flag.

            CHRGD
                Applied electric charge density.

            VLTG
                Applied voltage drop.

        key
            Symbol key:

            0
                Do not show body force load contours.

            1
                Show body force load contours.

            2
                Show current density as a vector (not a contour).

        Notes
        -----
        Shows body force loads as contours on displays for the selected
        elements.

        The effects of the ``mapdl.pbf`` command are not cumulative (that is, the
        command does not modify an existing setting from a previously issued
        ``mapdl.pbf`` command).  If you issue multiple ``mapdl.pbf`` commands
        during an analysis, only the setting specified by the most recent
        ``mapdl.pbf`` command applies.

        Use ``mapdl.pstatus()`` or ``mapdl.pbf('STAT')`` to display settings.
        Use ``mapdl.pbf('DEFA')`` to reset all specifications back to default.
        See also the ``mapdl.psf`` and ``mapdl.pbc`` command
        for other display contours.

        Portions of this command are not supported by PowerGraphics
        [``mapdl.graphics('POWER')``].

        This command is valid in any processor.

        .. note:
            In APDL the /PBF command has an unused 2nd argument. This has been
            removed in PyMAPDL because of its redundancy, and should be
            kept in mind when translating between the two.

        Examples
        --------
        Activate display of body loads of structural temperature on subsequent plots
        by showing body force and contours.

        >>> mapdl.pbf('TEMP', 1)
        """
        return self.run(f"/PBF,{item},,{key}", **kwargs)

    def plopts(self, label="", key="", **kwargs):
        """Controls graphics options on subsequent displays.

        APDL Command: /PLOPTS

        Parameters
        ----------
        label
            Apply display items as selected from the following labels:

            INFO
                Controls the display of the legend (ON or OFF) and allows the choice of preset
                or Multi-legend placement. Control is provided by the KEY
                values. (Defaults to KEY=3 when the GUI is on. Defaults to
                KEY= 2 otherwise.)

            LEG1
                Header portion of legend column (defaults to ON).

            LEG2
                View portion of legend column (defaults to ON (except off with contour
                displays)).

            LEG3
                View the contour section of the legend column (defaults to ON).

            FRAME
                Frame border lines around windows (defaults to ON).

            TITLE
                Title (bottom left text) (defaults to ON).

            MINM
                Min-Max symbols on contour displays (defaults to ON).

            LOGO
                ANSYS logo (defaults to OFF (displayed as text at top of legend column)).  If
                KEY = ON, the text is removed from legend column but the
                logo symbol is displayed in whichever active window is
                either in the uppermost right corner and on top, or if there
                is no window in that location, then in the window to the
                furthest right of the screen.  Version information remains
                in the legend column.

            WINS
                Controls whether graphics windows automatically stretch or shrink to adjust to
                screen size as the legend column is turned off or on
                [/PLOPTS,INFO] (defaults to ON).  If WINS is on and the
                legend column is changed from off to on, all windows are
                shrunk regardless of what their correct size is.

            WP
                Working plane (defaults to OFF).  The working plane is drawn as part of the
                display (not just an overlaid image as in WPSTYL).  This
                option is best used in combination with a hidden-line
                technique [/TYPE].

            DATE
                Controls the display of the date and time in your legend. Subsequent KEY values
                control the display as follows:

            FILE
                Controls the display of the ANSYS jobname in your legend. Subsequent KEY values
                control the display as follows:

        key
            Switch:

            OFF or 0
                Do not apply this display item. For Label = DATE, no time or date are
                displayed.

            ON or 1
                Apply this display item. For Label = DATE, show only the date.

            AUTO or 2
                For Label = INFO, initiate Auto-legend mode.  If the display has contours, the
                legend is ON; if the display has no contours, the
                legend is OFF. For Label = DATE, display both the date
                and time.

            3
                For Label = INFO , switch to Multi-legend mode. See the /UDOC command for the
                available legend configurations.

        Notes
        -----
        Use /PLOPTS,STAT to display settings.  Use /PLOPTS,DEFA to reset all
        specifications back to their defaults.

        When you perform multiple results displays, contours on the legend
        column may be truncated.  To avoid this, specify /PLOPTS,LEG1,0.

        The Multi-legend mode provides a number of legend data item priority
        and placement options. These options are accessed via the GUI at
        Utility Menu> PlotCtrls> Window Controls> Window Options. The /UDOC
        command provides command line options for this capability.

        This command is valid in any processor.

        This command is not available for Academic Research or Teaching level
        products
        """
        command = f"/PLOPTS,{label},{key}"
        return self.run(command, **kwargs)

    def pnum(self, label="", key="", **kwargs):
        """Controls entity numbering/coloring on plots.

        APDL Command: /PNUM

        Parameters
        ----------
        label
            Type of numbering/coloring:

            NODE
                Node numbers on node and element plots.

            ELEM
                Element numbers and colors on element plots.

            SEC
                Section numbers and colors on element and solid model plots (see "Notes").

            MAT
                Material set numbers and colors on element and solid model plots  (see
                "Notes").

            TYPE
                Element type reference numbers and colors on element and solid model plots (see
                "Notes").

            REAL
                Real constant set numbers and colors on element and solid model plots (see
                "Notes").

            ESYS
                Element coordinate system numbers on element and solid model plots (see
                "Notes").

            PART
                Element part numbers and colors on element plots (applicable to ANSYS LS-DYNA
                only).

            LOC
                Location numbers/colors of the element in matrix assembly order on element
                plots.

                .. note:: ``LOC`` and ``ELEM`` numbers will be the same unless the model has been reordered.

            KP
                Keypoint numbers on solid model plots.

            LINE
                Line numbers on solid model plots (both numbers and colors on line plots).

            AREA
                Area numbers on solid model plots (both numbers and colors on area plots).

            VOLU
                Volume numbers on solid model plots (both numbers and colors on volume plots).

            SVAL
                Stress (or any contour) values on postprocessing plots, and
                surface load values and colors on model plots (when surface load
                symbols are on [/PSF]).  For tabular boundary conditions, the
                table-evaluated values will be displayed on node, element, or
                contour displays in POST1 when load symbols (/PBF, /PSF, /PBC)
                are on and TABNAM is OFF.

            TABNAM
                Table names for tabular boundary conditions.  If this label is
                turned on, the table name appears next to the appropriate
                symbol, arrow, face outline, or contour as dictated by the /PSF,
                /PBC, and /PBF commands.

            STAT
                Shows current settings for /PNUM.

            DEFA
            key
            Switch:

            0
                Turns OFF numbers/colors for specified label.

            1
                Turns ON numbers/colors for specified label.

        Notes
        -----
        This command specifies entity numbering and coloring for subsequent
        plots.

        The MAT, TYPE, REAL, and ESYS labels activate both the numbering and
        coloring of the corresponding attributes for  EPLOT, KPLOT, LPLOT,
        APLOT, and VPLOT. The ELEM, MAT, TYPE, REAL, ESYS, PART (ANSYS LS-DYNA
        only), and LOC labels are mutually exclusive, i.e., only one can be
        specified at a time. Also, turning on a LINE, AREA, or VOLU label will
        turn off the MAT, TYPE, REAL, and PART labels.

        PowerGraphics [/GRAPHICS,POWER] displays for/PNUM can be problematic.
        /PNUM,ELEM will display erratically depending on other display command
        specifications, while /PNUM,LOC and /PNUM,ESYS are not supported.

        Element and volume numbers are not visible for 3-D elements and volumes
        when Z-buffering is turned on (/TYPE,,[6,7, or 8]).

        Use /PSTATUS or /PNUM,STAT to show settings. Use /PNUM,DEFA to reset
        all specifications back to default. Use the /NUMBER command to control
        whether numbers and colors are displayed together.

        This command is valid in any processor
        """
        command = f"/PNUM,{label},{key}"
        return self.run(command, **kwargs)

    def psf(self, item="", comp="", key="", kshell="", color="", **kwargs):
        """Shows surface load symbols on model displays.

        APDL Command: /PSF

        Parameters
        ----------
        item, comp
            Labels identifying the surface load to be shown; see
            Table 227: /PSF Valid Item and Component Labels.

        key
            Key to turn surface load symbols on or off:

            0
                Off (default).

            1
                On, shown as face outlines.  Line surface loads (SFL) on solid model plots are
                shown as arrows.

            2
                On, shown as arrows.

            3
                On, shown as color filled surfaces.  Line and area surface loads (SFL and SFA)
                on solid model plots are shown as arrows.

        kshell
            Visibility key for shell elements.

            0
                Off (default),  surface load symbols are displayed only on visible load faces.

            1
                On, surface load symbols are displayed even if load face is not visible.

        color
            Visibility key for contour legend.

            ON
                The symbols (arrows or face outlines) will show up in color with the legend
                 showing the corresponding color labels (default).

            OFF
                The contour legend will not be displayed. The symbols (arrows or face outlines)
                  will show up in grey. The size of the arrows will be
                  proportional to the applied load.

        Notes
        -----
        The ``mapdl.psf`` command determines whether and how to show surface loads on
        subsequent model displays.

        If surface loads are applied to solid model entities, only solid model
        plots show the load symbols; node and element plots do not show them
        unless the loads are transferred (SFTRAN or SBCTRAN).  Similarly, solid
        model plots do not show the load symbols if surface loads are applied
        to nodes and elements.  For node and element plots of shell element
        models, the surface load symbols are shown only if the load face is
        visible from the current viewing direction.

        The effects of the ``mapdl.psf`` command are not cumulative (that is, the
        command does not modify an existing setting from a previously issued
        ``mapdl.psf`` command).  Only the setting specified via the most recent
        ``mapdl.psf`` command applies.

        If you issue a postprocessing (``mapdl.post1``) plot command that produces result
        contours (such as PLNSOL), the ``mapdl.psf`` command has no effect. This
        behavior prevents conflicting contours in the graphics window.

        When using the radiosity method (Item = RDSF and Comp = ENCL) with Key
        = 2, the radiation arrows point outward from any element face. When
        using SURF154 with KEYOPT(2) = 1, set the Item to PRES and leave the
        Component Label blank.

        ``mapdl.psf('STAT')`` displays current ``mapdl.psf`` settings, and
        ``mapdl.psf('DEFA')`` resets them back to default.

        Other useful commands are ``mapdl.pnum('SVAL', 1)`` to show the values of the
        surface loads, ``mapdl.vscale()`` to change arrow lengths, and
        ``mapdl.pbc`` and ``mapdl.pbf`` to activate other load symbols.

        For beam elements, only the colors representing shear (GREEN) and
        normal (RED) pressures are displayed for the arrows. The color of these
        arrows does not correspond to the magnitudes in the contour legend. The
        length of these arrows does, however, correlate to the relative
        magnitude of the pressures.

        For elements SURF159, SOLID272, SOLID273, PIPE288 and PIPE289, the /PSF
        command is not available when displaying elements with shapes
        determined from the real constants or section definition (``mapdl.eshape``).
        For PIPE288 and PIPE289, only external loads applied via the
        ``mapdl.sfbeam`` command are displayed.

        This command is valid in any processor.

        Table: 227:: : /PSF
            Valid Item and Component Labels

        Pressure loads apply to the element coordinate system (KEYOPT(2) = 0).
        Adjust appropriately for a local coordinate system (KEYOPT(2) = 1). See
        Figure: 153.2:: Pressures in the Element Reference.

        Examples
        --------
        On subsequent plots display the surface loads of pressure as arrows.

        >>> mapd.psf('PRES', '', 2)

        Activate display of convection on surfaces using the element outline option.

        >>> mapdl.psf('CONV', '', 1)
        """
        command = f"/PSF,{item},{comp},{key},{kshell},{color}"
        return self.run(command, **kwargs)

    def psymb(self, label="", key="", **kwargs):
        """Shows various symbols on displays.

        APDL Command: /PSYMB

        Parameters
        ----------
        label
            Show symbols as selected from the following labels:

            CS
                Local coordinate systems.

            NDIR
                Nodal coordinate systems (on rotated nodes only).

            ESYS
                Element coordinate systems (element displays only).

            LDIR
                Line directions (line displays only).

            LDIV
                Controls the display of element divisions on lines.

            ADIR
                Area direction symbol (for keypoint, line, area and volume plots).

            LAYR
                Layer orientations (relative to the projected element x-axis) or fiber
                orientations in smeared reinforcing elements. Used only
                within an element display. Use KEY to specify the layer
                number.

            ECON
                Element mesh symbols on keypoints and lines.

            DOT
                Larger symbols displayed for node and keypoint locations.  When Label = DOT,
                KEY = 1 by default.

            XNOD
                Extra node of surface or circuit elements.

            FBCS
                Force boundary condition scaling. Subsequent KEY value determines whether or
                not to scale the applied and derived forces/moments to the
                same maximum value.

            DEFA
                Resets the symbol keys so that ANSYS displays none of the symbols controlled by
                the /PSYMB command.  The value of the KEY field is ignored.

            STAT
                Prints the status of the settings of the symbol
                keys controlled by the /PSYMB command.  The KEY
                field is ignored.

        key
            Symbol key:

            ``-1``
                Effective only if Label = LAYR and solid shape
                element display (/ESHAPE) is active. Orientation of
                all layers appears with the solid shape element
                display.

            0
                No symbol (default). If Label = LDIV, then KEY= 0
                indicates that the displayed element divisions will
                correspond to the existing mesh (the word MESHED or
                EXISTING can also be substituted). Also, for Label =
                LDIV, if you execute any meshing command (such as
                AMESH or VMESH), KEY is set to 0 (MESHED)
                automatically. If Label = FBCS, then KEY= 0 indicates
                that boundary condition scaling will not be
                common. The applied and derived forces/moments will
                be scaled to their respective maximum values.

            1
                Include symbol. If Label = LDIV, then KEY = 1
                indicates that the displayed line divisions will
                correspond to the value assigned by LESIZE (the word
                LESIZE can also be substituted). Also, for Label =
                LDIV, if you execute the LESIZE command, KEY is set
                to 1 (LESIZE) automatically. If Label = FBCS, then
                KEY= 1 indicates that boundary condition scaling will
                be common. The applied and derived forces/moments
                will be scaled to the same maximum value.

            N
                If Label = LAYR, then N is equal to the layer
                number. If Label = DOT, then N can be equal to
                0,1,.....15, indicating the dot size. If Label =
                LDIV, then KEY = -1, indicates that no element
                divisions will be displayed (the word OFF can also be
                substituted).

        Notes
        -----
        Includes various symbols on the display.  Triads are right-handed with
        x displayed as the longest leg.  Where color is displayed, x is white,
        y is green, and z is blue.  For beams, x is always along the length of
        the element.   For lines, an arrow represents the direction of a line,
        from the beginning keypoint to the end keypoint.  See /PLOPTS command
        for additional display options.  Use /PSTATUS or /PSYMB,STAT to display
        settings.  Use /PSYMB,DEFA to reset all specifications back to their
        defaults.  The command /PSYMB,ECON,1 causes the symbol "M" to be
        displayed on keypoints and lines associated with meshed entities.  When
        you issue the command /PSYMB,DOT,1, a larger symbol is displayed for
        each node and keypoint location.

        PowerGraphics (/GRAPHICS,POWER) does not support /PSYMB,ESYS and
        /PSYMB,LAYR.

        If KEY = N and PowerGraphics is off, the centroid of the surface
        elements is connected to the extra node using a gray line.  However, if
        PowerGraphics is on, the color of the line connecting the centroid to
        the extra node is the same as that for the elements themselves (as
        determined by /PNUM).

        When Label = LAYR, the layer systems can be visualized with all
        current-technology layered elements and the smeared reinforcing element
        REINF265. To use /PSYMB,LAYR with REINF265, first set the vector-mode
        graphics option (/DEVICE,VECTOR,1).

        This command is valid in any processor.

        """
        command = f"/PSYMB,{label},{key}"
        return self.run(command, **kwargs)

    def triad(self, lab="", **kwargs):
        """Shows the global XYZ coordinate triad on displays.

        APDL Command: /TRIAD

        Parameters
        ----------
        lab
            Display triad as follows:

            ORIG
                Display triad at global origin (default).

            OFF
                Turn off triad display.

            LBOT
                Display triad in lower left screen corner.

            RBOT
                Display triad in lower right screen corner.

            LTOP
                Display triad in upper left screen corner.

            RTOP
                Display triad in upper right screen corner.

        Notes
        -----
        For efficiency, ANSYS 3-D graphics logic maintains a single data
        structure (segment), which includes the triad as a 3-D data object.  If
        a 3-D device is involved (/SHOW,3D), and the ANSYS graphics are not
        being displayed as multi-plots, then the triad location is determined
        by the view settings for Window #1.  A request for triad display
        anywhere except for the origin could yield an improper display in
        windows 2 through 5. The program displays the same segment in all
        windows.  The view settings of each window constitute the only
        difference in the display in the active windows.

        This command is valid in any processor.
        """
        command = f"/TRIAD,{lab}"
        return self.run(command, **kwargs)

    def udoc(self, wind="", cl_ass="", key="", **kwargs):
        """Determines position and content for the multi-legend options.

        APDL Command: /UDOC

        Parameters
        ----------
        wind
            The window number to which the command applies. (defaults to 1)

        class
            The type (and relative importance) of legend item being displayed:

            CNTR
                Contour legend. This legend item is controlled separately from the other legend
                items (see note below).

            DATE
                The items in the DATE class include the date and time, or the ANSYS graphical
                logo (/PLOPTS,LOGO,1). This item is shown by default in all
                plots.

            GWIN
                The items in the GWIN class include the entity acronyms that appear in the
                legend of a multiplot of entities (Nodes, Elements,
                Keypoints, Lines, Areas, Volumes). GWIN items are shown by
                default for all GPLOT displays.

            TYPE
                Items in the TYPE class include the plot type (e.g. ELEMENTS, MATERIALS, NODAL
                SOLUTIONS, etc.). TYPE items are shown by default in all
                plots.

            TYP2
                Items in the TYP2 class include supplementary type information, such as DMAX
                and SMAX for nodal solutions. TYP2 items are shown by
                default in all plots.

            INUM
                Items in the INUM class include the number labels generated by the /PNUM
                command. This class is displayed by default in all plots
                that contain /PNUM information.

            BCDC
                The items in the BCDC class include labels created by the /PBC  command. This
                class is shown by default in all plots which contain /PBC
                information.

            VECT
                Items in the VECT class include labels created by the PLVECT command. This
                class is shown by default for all PLVECT plots.

            SURF
                The items in the SURF class include labels from the /PSF legend. This class is
                shown by default on all plots of surface boundary
                conditions.

            BODY
                Items from the BODY class include labels from the /PBF legend. This class is
                shown by default in all plots of body forces.

            PSTA
                Items from the PSTA class include stress scaling statistics, such as the
                /SSCALE setting. This class is not shown as the default for
                any type of plot, and must be specifically referenced to
                display the included data.

            VIEW
                The items in the VIEW class include view statistics. This class is not shown as
                the default for any type of plot, and must be specifically
                referenced to display the included data.

            MISC
                The items in the MISC class include supplementary labels like /EXPANDED and
                Stress Section Cross Section. This class is not shown as the
                default for any type of plot, and must be specifically
                referenced to display the included data.

        key
            Switch:

        Notes
        -----
        The legend classes conform to the controls specified in the window
        options panel (PlotCtrls> Window Controls> Window Options). In many
        instances, the legend controls specified with the /PLOPTS command will
        take precedence and override /UDOC specifications. For instance:

        /PLOPTS,LEG1,OFF will disable the TYPE, TYP2, INUM, and MISC classes,
        regardless of the /UDOC settings.

        /PLOPTS,LEG2,OFF will disable the VIEW class, regardless of the /UDOC
        settings.

        /PLOPTS,LEG3,OFF will disable the PSTA class, regardless of the /UDOC
        settings.

        All items in a class are listed with the same X coordinate (except for
        contours). The contents of the text classes are dumped onto the display
        window from top to bottom, in order of class importance.

        The font specification for text items that are included in the user-
        specified legends are controlled with the /DEVICE command (PlotCtrls>
        Font Controls> Anno/Graph Font).

        The floating point values for the data presented in the legend(s) are
        controlled by the /GFORMAT command.
        """
        command = f"/UDOC,{wind},{cl_ass},{key}"
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from typing import Optional, Protocol

from ansys.mapdl.core.mapdl import MapdlBase


class _Backend(Protocol):
    def run_command(self) -> str: ...


class MapdlInProcess(MapdlBase):
    def __init__(self, backend: _Backend):
        super().__init__(
            loglevel="WARNING", use_vtk=False, log_apdl=None, print_com=False
        )
        self._backend = backend
        self._cleanup: bool = True
        self._name: str = "MapdlInProcess"
        self._session_id: Optional[str] = None
        self._mute: bool = False

    def _run(self, command: str, verbose: bool = False, mute: bool = False) -> str:
        if not command.strip():
            raise ValueError("Empty commands not allowed")

        if len(command) > 639:
            raise ValueError("Maximum command length mut be less than 640 characters")

        return self._backend.run_command(command, verbose, mute).strip()

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, name) -> None:
        self._name = name

    def _check_session_id(self) -> None:
        pass

    def __repr__(self):
        info = super().__repr__()
        return info



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class FailureCriteria:
    def fc(
        self,
        mat="",
        lab1="",
        lab2="",
        data1="",
        data2="",
        data3="",
        data4="",
        data5="",
        data6="",
        **kwargs,
    ):
        """Provides failure criteria information and activates a data

        APDL Command: FC
        table to input temperature-dependent stress and strain limits.

        Parameters
        ----------
        mat
            Material reference number. You can define failure criteria
            for up to 250 different materials.

        lab1
            Type of data.

            TEMP - Temperatures. Each of the materials you define can
            have a different set of temperatures to define the failure
            criteria.

            EPEL - Strains.

            S - Stresses.

        lab2
            Specific criteria. Not used if Lab1 = TEMP.

            XTEN - Allowable tensile stress or strain in the
            x-direction. (Must be positive.)

            XCMP - Allowable compressive stress or strain in the
            x-direction. (Defaults to negative of XTEN.)

            YTEN - Allowable tensile stress or strain in the
            y-direction. (Must be positive.)

            YCMP - Allowable compressive stress or strain in the
            y-direction. (Defaults to negative of YTEN.)

            ZTEN - Allowable tensile stress or strain in the
            z-direction. (Must be positive.)

            ZCMP - Allowable compressive stress or strain in the
            z-direction. (Defaults to negative of ZTEN.)

            XY - Allowable XY stress or shear strain. (Must be
            positive.)

            YZ - Allowable YZ stress or shear strain. (Must be
            positive.)

            XZ - Allowable XZ stress or shear strain. (Must be
            positive.)

            XYCP - XY coupling coefficient (Used only if Lab1 =
            S). Defaults to -1.0. [1]

            YZCP - YZ coupling coefficient (Used only if Lab1 =
            S). Defaults to -1.0. [1]

            XZCP - XZ coupling coefficient (Used only if Lab1 =
            S). Defaults to -1.0. [1]

        data1, data2, data3, . . . , data6
            Description of DATA1 through DATA6.

            T1, T2, T3, T4, T5, T6 - Temperature at which limit data
            is input. Used only when Lab1 = TEMP.

            V1, V2, V3, V4, V5, V6 - Value of limit stress or strain
            at temperature T1 through T6. Used only when Lab1 = S or
            EPEL.

        Notes
        -----
        The data table can be input in either PREP7 or POST1. This
        table is used only in POST1. When you postprocess failure
        criteria results defined via the FC command (PLESOL, PRESOL,
        PLNSOL, PRNSOL, PRRSOL, etc.), the active coordinate system
        must be the coordinate system of the material being
        analyzed. You do this using RSYS, SOLU. For layered
        applications, you also use the LAYER command. See the specific
        element documentation in the Element Reference for information
        on defining your coordinate system for layers.

        Some plotting and printing functions will not support Failure
        Criteria for your PowerGraphics displays. This could result in
        minor changes to other data when Failure Criteria are
        applied. See the appropriate plot or print command
        documentation for more information.
        """
        command = "FC,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (
            str(mat),
            str(lab1),
            str(lab2),
            str(data1),
            str(data2),
            str(data3),
            str(data4),
            str(data5),
            str(data6),
        )
        return self.run(command, **kwargs)

    def fccheck(self, **kwargs):
        """Checks both the strain and stress input criteria for all materials.

         APDL Command: FCCHECK

        Notes
        -----
        Issue the FCCHECK command to check the completeness of the input during
        the input phase.
        """
        command = "FCCHECK,"
        return self.run(command, **kwargs)

    def fcdele(self, mat="", **kwargs):
        """Deletes previously defined failure criterion data for the given

        APDL Command: FCDELE
        material.

        Parameters
        ----------
        mat
            Material number. Deletes all FC command input for this material.

        Notes
        -----
        This command is also valid in POST1.
        """
        command = "FCDELE,%s" % (str(mat))
        return self.run(command, **kwargs)

    def fclist(self, mat="", temp="", **kwargs):
        """To list what the failure criteria is that you have input.

        APDL Command: FCLIST

        Parameters
        ----------
        mat
             Material number (defaults to ALL for all materials).

        temp
            Temperature to be evaluated at (defaults to TUNIF).

        Notes
        -----
        This command allows you to see what you have already input for failure
        criteria using the FC commands.
        """
        return self.run(f"FCLIST,{mat},,{temp}", **kwargs)

    def fctyp(self, oper="", lab="", **kwargs):
        """Activates or removes failure-criteria types for postprocessing.

        APDL Command: FCTYP

        Parameters
        ----------
        oper
            Operation key:

            ADD - Activate failure-criteria types. This option is the default behavior.

            DELE - Remove failure-criteria types.

        lab
            Valid failure-criteria labels. If ALL, select all available
            (including user-defined) failure criteria.

            EMAX - Maximum strain criterion (default)

            SMAX - Maximum stress criterion (default)

            TWSI  - Tsai-Wu strength index (default)

            TWSR  - Inverse of Tsai-Wu strength ratio index (default)

            HFIB  - Hashin fiber failure criterion

            HMAT  - Hashin matrix failure criterion

            PFIB  - Puck fiber failure criterion

            PMAT  - Puck inter-fiber (matrix) failure criterion

            L3FB - LaRc03 fiber failure criterion

            L3MT - LaRc03 matrix failure criterion

            L4FB - LaRc04 fiber failure criterion

            L4MT - LaRc04 matrix failure criterion

            USR1 through USR9  - User-defined failure criteria

        Notes
        -----
        The FCTYP command modifies the list of active failure criteria.

        By default, active failure criteria include EMAX, SMAX, TWSI, and TWSR.

        The command affects any subsequent postprocessing listing and plotting
        commands (such as PRESOL, PRNSOL, PLESOL, PLNSOL, and ETABLE).

        A single FCTYP command allows up to six failure-criteria labels. If
        needed, reissue the command to activate or remove additional failure-
        criteria types.
        """
        command = f"FCTYP,{oper},{lab}"
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class MatrixOP:
    def axpy(self, vr="", vi="", m1="", wr="", wi="", m2="", **kwargs):
        """Performs the matrix operation ``M2= v*M1 + w*M2``.

        APDL Command: ``*AXPY``

        Parameters
        ----------
        vr, vi
            The real and imaginary parts of the scalar v. Default value is 0.

        m1
            Name of matrix M1. If not specified, the operation ``M2 = w*M2`` will
            be performed.

        wr, wi
            The real and imaginary parts of the scalar w. Default value is 0.

        m2
            Name of matrix M2. Must be specified.

        Notes
        -----
        The matrices M1 and M2 must have the same dimensions and same type
        (dense or sparse). If M2 is real, vi and wi are ignored.
        """
        command = f"*AXPY,{vr},{vi},{m1},{wr},{wi},{m2}"
        return self.run(command, **kwargs)

    def comp(self, matrix="", algorithm="", threshold="", **kwargs):
        """Compresses the columns of a matrix using a specified algorithm.

        APDL Command: ``*COMP``

        Parameters
        ----------
        matrix
            Name of the matrix to compress.

        algorithm
            Algorithm to use:

            Singular value decomposition algorithm (default). - Modified Gram-Schmidt algorithm.

        threshold
            Numerical threshold value used to manage the compression. Default
            value for SVD is 1E-7; default value for MGS is 1E-14.

        Notes
        -----
        The algorithms available through this command are only applicable to
        dense matrices that were created using the ``*DMAT`` command.

        Columns which are linearly dependent on others are removed, leaving the
        independent or basis vectors. The matrix is resized according to the
        new size determined by the algorithm.
        """
        command = f"*COMP,{matrix},{algorithm},{threshold}"
        return self.run(command, **kwargs)

    def dmat(
        self,
        matrix="",
        type_="",
        method="",
        val1="",
        val2="",
        val3="",
        val4="",
        val5="",
        **kwargs,
    ):
        """Creates a dense matrix.

        APDL Command: ``*DMAT``

        Parameters
        ----------
        matrix
            Name used to identify the matrix. Must be specified.

        type\\_
            Matrix type:

            Double precision real values (default). - Complex double precision values.

        method
            Method used to create the matrix:

            Allocate space for a matrix (default). - Resize an
            existing matrix to new row and column dimensions. Values
            are kept from the original matrix. If the dimensions
            specified by Val1 (rows) and Val2 (columns) are greater
            than the original matrix size, the additional entries are
            assigned a value of zero.

            Copy an existing matrix. - Link to an existing matrix. The
            memory will be shared between the original matrix and the
            new matrix. This is useful for manipulating a submatrix of
            a larger matrix. The Val1 through Val5 arguments will be
            used to specify the lower and upper bounds of row and
            column numbers from the original matrix.

        val1, val2, val3, val4, val5
            Additional input. The meaning of Val1 through Val5 will vary
            depending on the specified Method. See details below.

        Notes
        -----
        This command allows you to create a dense matrix. To create a sparse
        matrix, use the ``*SMAT`` command. ``*SMAT`` is recommended for large matrices
        obtained from the .FULL or .HBMAT file. Refer to the HBMAT command
        documentation for more information about .FULL file contents.

        Use the ``*VEC`` command to create a vector.

        For very large matrices, use the OUTOFCORE option (Method = ALLOC or
        COPY) to keep some of the matrix on disk if there is insufficient
        memory.

        When importing a dense matrix from a DMIG file, you can define the
        formatting of the file using the Val3 and Val4 fields.

        """
        command = f"*DMAT,{matrix},{type_},{method},{val1},{val2},{val3},{val4},{val5}"
        return self.run(command, **kwargs)

    def dot(self, vector1="", vector2="", par_real="", par_imag="", **kwargs):
        """Computes the dot (or inner) product of two vectors.

        APDL Command: ``*DOT``

        Parameters
        ----------
        vector1
            Name of first vector; must have been previously specified by a ``*VEC``
            command.

        vector2
            Name of second vector; must have been previously specified by a
            ``*VEC`` command.

        par_real
            Parameter name that contains the result.

        par_imag
            Parameter name that contains the imaginary part of the result (used
            only for complex vectors).

        Notes
        -----
        If Vector1 and Vector2 are complex, the complex conjugate of Vector1 is
        used to compute the result (Par_Real, Par_Imag).
        """
        command = f"*DOT,{vector1},{vector2},{par_real},{par_imag}"
        return self.run(command, **kwargs)

    def eigen(self, kmatrix="", mmatrix="", cmatrix="", evals="", evects="", **kwargs):
        """Performs a modal solution with unsymmetric or damping matrices.

        APDL Command: ``*EIGEN``

        Parameters
        ----------
        kmatrix
            Name of the stiffness matrix. May be a real or complex-valued
            matrix.

        mmatrix
            Name of the mass matrix.

        cmatrix
            Name of the damping matrix (used only for MODOPT,DAMP).

        evals
            Name of the output eigenvalues vector. It will be an m-long ``*VEC``
            vector of complex values, where m is the number of eigenvalues
            requested (MODOPT).

        evects
            Name of the output eigenvector matrix. It will be a n x m ``*DMAT``
            (dense) matrix of complex values, where n is the size of the matrix
            and m is the number of eigenvalues requested (MODOPT).

        Notes
        -----
        Use the command ANTYPE,MODAL and the MODOPT command to specify the
        modal solution options. Only MODOPT,DAMP, MODOPT,UNSYM, MODOPT,LANB,
        and MODOPT,SUBSP are supported.

        ``*EIGEN`` with Block Lanczos (LANB) only supports sparse matrices.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"*EIGEN,{kmatrix},{mmatrix},{cmatrix},{evals},{evects}"
        return self.run(command, **kwargs)

    def export(
        self,
        matrix="",
        format_="",
        fname="",
        val1="",
        val2="",
        val3="",
        **kwargs,
    ):
        """Exports a matrix to a file in the specified format.

        APDL Command: ``*EXPORT``

        Parameters
        ----------
        matrix
            Name of the matrix to export (must be a matrix previously created
            with ``*DMAT`` or ``*SMAT``, or a vector previously created with ``*VEC``).

        format\\_
            Format of the output file:

            Export the matrix in the Matrix Market Format. - Export
            the matrix in the SUB file format.

            Export the matrix in the Harwell-Boeing file format. -
            Export the matrix in a native format, to be re-imported
            using the ``*DMAT`` or ``*SMAT`` command.

            Export the matrix to an existing EMAT file. - Export the
            matrix to an APDL array parameter.

            Export the matrix profile to a Postscript file. - Export
            the matrix in the DMIG file format.

        fname
            Name of the file, or name of the array parameter if Format = APDL.

        val1, val2, val3
            Additional input. The meaning of Val1 through Val3 will vary
            depending on the specified Format. See table below for details.

        Notes
        -----
        Only sparse matrices can be exported to Postscript files. This option
        plots the matrix profile as a series of dots.

        If you want to create a .SUB file from several matrices, you need to
        set Val3 = WAIT for all matrices but the last, and Val3 = DONE for the
        last one. The export will be effective at the last ``*EXPORT`` command.

        To create a .SUB file or .DMIG file from scratch, you must supply the
        row information array. (Specify this array in the Val2 field for .SUB
        or in the Val1 field for .DMIG.) This must be an m x 2 array, where m
        is the size of the matrix. The first column is the node number and the
        second column is the DOF number corresponding to each row of the
        matrix.

        The ``*EXPORT`` command is not applicable to sparse matrices initialized
        from .FULL files by means of the NOD2BCS option on the ``*SMAT`` command
        (i.e., ``*SMAT,,,IMPORT,FULL,,NOD2BCS``).
        """
        command = f"*EXPORT,{matrix},{format_},{fname},{val1},{val2},{val3}"
        return self.run(command, **kwargs)

    def fft(
        self,
        type_="",
        inputdata="",
        outputdata="",
        dim1="",
        dim2="",
        resultformat="",
        **kwargs,
    ):
        """Computes the fast Fourier transformation of a specified matrix or

        APDL Command: ``*FFT``
        vector.

        Parameters
        ----------
        type\\_
            Type of FFT transformation:

            Forward FFT computation (default). - Backward FFT computation.

        inputdata
            Name of matrix or vector for which the FFT will be computed. This
            can be a dense matrix (created by the ``*DMAT`` command) or a vector
            (created by the ``*VEC`` command). Data can be real or complex values.
            There is no default value for this argument.

        outputdata
            Name of matrix or vector where the FFT results will be stored. The
            type of this argument must be consistent with InputData (see table
            below). There is no default value for this argument.

        dim1
            The number of terms to consider for a vector, or the number of rows
            for a matrix. Defaults to the whole input vector or all the rows of
            the matrix.

        dim2
            The number of columns to consider for a matrix. Defaults to all the
            columns of the matrix. (Valid only for matrices.)

        resultformat
            Specifies the result format:

            Returns the full result. That is, the result matches the
            dimension specified on this command (DIM1, DIM2). -
            Returns partial results. For real input data, there is a
            symmetry in the results of the Fourier transform as some
            coefficients are conjugated. The partial format uses this
            symmetry to optimize the storage of the results. (Valid
            only for real data.)

        Notes
        -----
        In the example that follows, the fast Fourier transformation is used to
        filter frequencies from a noisy input signal.
        """
        command = f"*FFT,{type_},{inputdata},{outputdata},{dim1},{dim2},{resultformat}"
        return self.run(command, **kwargs)

    def free(self, name="", **kwargs):
        """Deletes a matrix or a solver object and frees its memory allocation.

        APDL Command: ``*FREE``

        Parameters
        ----------
        name
            Name of the matrix or solver object to delete. Use Name = ALL to
            delete all APDL Math matrices and solver objects.  Use Name = WRK
            to delete all APDL Math matrices and solver objects that belong to
            a given workspace.

        val1
            If Name = WRK, Val1 is to set the memory workspace number.

        Notes
        -----
        A /CLEAR command will automatically delete all the current APDL Math
        objects.
        """
        command = f"*FREE,{name}"
        return self.run(command, **kwargs)

    def init(self, name="", method="", val1="", val2="", val3="", **kwargs):
        """Initializes a vector or dense matrix.

        APDL Command: ``*INIT``

        Parameters
        ----------
        name
            Vector or matrix which will be initialized. This can be a vector
            (created by the ``*VEC`` command) or a dense matrix (created by the
            ``*DMAT`` command).

        method
            Initialization method to use:

            Fill the vector/matrix with zeros (default). - Fill the vector/matrix with a constant value.

            Fill the vector/matrix with random values. - Fill the nth diagonal of the matrix with a constant value. Other values are not
                              overwritten.

        val1, val2, val3
            Additional input. The meaning of Val1 through Val3 will vary
            depending on the specified Method. See details below.

        Notes
        -----
        This command initializes a previously defined vector (``*VEC``) or dense
        matrix (``*DMAT``).
        """
        command = f"*INIT,{name},{method},{val1},{val2},{val3}"
        return self.run(command, **kwargs)

    def itengine(
        self,
        type_="",
        enginename="",
        precondname="",
        matrix="",
        rhsvector="",
        solvector="",
        maxiter="",
        toler="",
        **kwargs,
    ):
        """Performs a solution using an iterative solver.

        APDL Command: ``*ITENGINE``

        Parameters
        ----------
        type\\_
            Specifies the algorithm to be used:

        enginename
            Name used to identify this iterative solver engine. Must be
            specified.

        precondname
            Linear solver engine name (``*LSENGINE``) identifying the factored
            matrix to be used as the preconditioner.

        matrix
            Name of the matrix to solve.

        rhsvector
            Matrix (load vector) name.

        solvector
            Solution vector name. If non-zero, it will be taken as the initial
            vector for the iterative process.

        maxiter
            Maximum number of iterations allowed. Default is 2 times the number
            of rows in the matrix.

        toler
            Convergence tolerance. Default is 1.0E-8.

        Notes
        -----
        This command solves Ax = b using a preconditioned conjugate gradient
        algorithm. It uses an existing factored system as the preconditioner.
        This solution method is useful if an existing matrix has been solved
        and minor changes have been made to the matrix.
        """
        command = f"*ITENGINE,{type_},{enginename},{precondname},{matrix},{rhsvector},{solvector},{maxiter},{toler}"
        return self.run(command, **kwargs)

    def lsbac(self, enginename="", rhsvector="", solvector="", **kwargs):
        """Performs the solve (forward/backward substitution) of a
        factorized linear system.

        APDL Command: ``*LSBAC``

        Parameters
        ----------
        enginename
            Name used to identify this engine. Must have been previously
            created using ``*LSENGINE`` and factorized using ``*LSFACTOR``.

        rhsvector
            Name of vector containing the right-hand side (load) vectors as
            input. Must have been previously defined as a ``*VEC`` vector or a
            ``*DMAT`` matrix.

        solvector
            Name of vector that will contain the solution vectors upon
            completion. Must be predefined as a ``*VEC`` vector or ``*DMAT`` matrix.

        Notes
        -----
        This command performs forward and back substitution to obtain the
        solution to the linear matrix equation Ax = b. The matrix engine must
        have been previously defined using ``*LSENGINE``, and the matrix factored
        using ``*LSFACTOR``.

        You can use the ``*DMAT,,,COPY`` (or ``*VEC,,,COPY``) command to copy the load
        vector to the solution vector in order to predefine it with the
        appropriate size.
        """
        command = f"*LSBAC,{enginename},{rhsvector},{solvector}"
        return self.run(command, **kwargs)

    def lsdump(self, enginename="", filename="", **kwargs):
        """Dumps a linear solver engine to a binary File.

        APDL Command: ``*LSDUMP``

        Parameters
        ----------
        enginename
            Name used to identify this engine. Must have been previously
            created using ``*LSENGINE`` and factorized using ``*LSFACTOR``.

        filename
            Name of the file to create.

        Notes
        -----
        Dumps a previously factorized linear solver system to a binary file.
        Only LAPACK and BCS linear solvers can be used with this feature. The
        Linear Solver can later be restored with the ``*LSRESTORE`` command.

        A BCS Sparse Solver can be dumped only if uses the INCORE memory option
        (see BCSOPTION).
        """
        command = f"*LSDUMP,{enginename},{filename}"
        return self.run(command, **kwargs)

    def lsengine(self, type_="", enginename="", matrix="", option="", **kwargs):
        """Creates a linear solver engine.

        APDL Command: ``*LSENGINE``

        Parameters
        ----------
        type_
            Specifies the algorithm to be used:

            * ``"BCS"`` - Boeing sparse solver (default if applied to
                        sparse matrices).

            * ``"DSS"`` : MKL sparse linear solver (Intel Windows and
                        Linux systems only).

            * ``"LAPACK"`` : LAPACK dense matrix linear solver (default if
                           applied to dense matrices).

            * ``"DSP"`` : Distributed sparse solver.

        enginename
            Name used to identify this engine. Must be specified.

        matrix
            Name of the matrix to solve.

        option
            Option to control the memory mode of the DSS solver (used only
            if ``type='dss'``):

            * ``"INCORE"`` : In-core memory mode

            * ``"OUTOFCORE"`` : Out-of-core memory mode

        Notes
        -----
        This command creates a linear solver engine.

        The BCS, DSS, and DSP solvers can only be used with sparse matrices.
        For dense matrices, use the LAPACK solver.
        """
        command = f"*LSENGINE,{type_},{enginename},{matrix},{option}"
        return self.run(command, **kwargs)

    def lsfactor(self, enginename="", option="", **kwargs):
        """Performs the numerical factorization of a linear solver system.

        APDL Command: ``*LSFACTOR``

        Parameters
        ----------
        enginename
            Name used to identify this engine. Must have been previously
            created using ``*LSENGINE``.

        option
            Option to invert the matrix, used only with an LAPACK engine
            (``*LSENGINE,LAPACK``):

        Notes
        -----
        Performs the computationally intensive, memory intensive factorization
        of a matrix specified by ``*LSENGINE``, using the solver engine also
        specified by ``*LSENGINE``.
        """
        command = f"*LSFACTOR,{enginename},{option}"
        return self.run(command, **kwargs)

    def lsrestore(self, enginename="", filename="", **kwargs):
        """Restores a linear solver engine from a binary file.

        APDL Command: ``*LSRESTORE``

        Parameters
        ----------
        enginename
            Name used to identify this engine.

        filename
            Name of the file to read from.

        Notes
        -----
        Restores a previously dumped Linear Solver (see the ``*LSDUMP`` command).
        This Linear Solver can be used to solve a linear system using the
        ``*LSBAC`` command.
        """
        command = "*LSRESTORE,%s,%s" % (str(enginename), str(filename))
        return self.run(command, **kwargs)

    def merge(self, name1="", name2="", val1="", val2="", **kwargs):
        """Merges two dense matrices or vectors into one.

        APDL Command: ``*MERGE``

        Parameters
        ----------
        name1
            Name of the matrix or vector to extend.

        name2
            Name of the matrix or vector to be merged into ``name1``.

        val1
            If ``name1`` refers to a dense matrix created by the ``*DMAT``
            command then the column or row number indicating where the new values
            are to be inserted into the Name1 matrix.

            If ``name` refers to a vector created by ``*VEC`` then this is the
            row number indicating where the new values are to be inserted
            into the ``name1`` vector.

        val2
            Specifies how the ``name2`` matrix or vector is copied into
            the ``name1`` matrix.

            * ``"COL"`` : Insert the new values at the column location
              specified by ``val1`` (default).
            * ``"row"`` : Insert the new values at the row location
              specified by ``val1``.

        Notes
        -----
        ``merge`` can be used to add new columns or rows to a dense matrix
        that was created by the ``*DMAT`` command. In this case, ``name1`` must
        be the name of the dense matrix and ``name2`` must refer to a vector
        or another dense matrix.

        ``*MERGE`` can also be used to add new rows to a vector that was
         created by the ``*VEC`` command. In this case, ``name1`` and
         ``name2`` must both refer to vectors.

        In all cases, the values of the original matrix or vector are
        retained, and the matrix or vector is resized to accommodate the
        additional rows or columns.
        """
        return self.run(f"MERGE,{name1},{name2},{val1},{val2}", **kwargs)

    def mult(self, m1="", t1="", m2="", t2="", m3="", **kwargs):
        """Performs the matrix multiplication ``M3 = M1(T1)*M2(T2)``.

        APDL Command: ``*MULT``

        Parameters
        ----------
        m1
            Name of matrix M1. Must have been previously specified by a ``*DMAT``
            or ``*SMAT`` command.

        t1
            Transpose key. Set T1 = TRANS to use the transpose of M1. If blank,
            transpose will not be used.

        m2
            Name of matrix M2. Must have been previously specified by a ``*DMAT``
            command.

        t2
            Transpose key. Set T2 = TRANS to use the transpose of M2. If blank,
            transpose will not be used.

        m3
            Name of resulting matrix, M3. Must be specified.

        Notes
        -----
        The matrices must be dimensionally consistent such that the number of
        columns of M1 (or the transposed matrix, if requested) is equal to the
        number of rows of M2 (or the transposed matrix, if requested).

        You cannot multiply two sparse matrices with this command (that is, M1
        and M2 cannot both be sparse). The resulting matrix, M3, will always be
        a dense matrix, no matter what combination of input matrices is used
        (dense*sparse, sparse*dense, or dense*dense).
        """
        command = f"*MULT,{m1},{t1},{m2},{t2},{m3}"
        return self.run(command, **kwargs)

    def nrm(self, name="", normtype="", parr="", normalize="", **kwargs):
        """Computes the norm of the specified matrix or vector.

        APDL Command: ``*NRM``

        Parameters
        ----------
        name
            Matrix or vector for which the norm will be computed. This
            can be a dense matrix (created by the ``*DMAT`` command),
            a sparse matrix (created by the ``*SMAT`` command) or a
            vector (created by the ``*VEC`` command)

        normtype
            Mathematical norm to use:

            - L2 (Euclidean or SRSS) norm (default).
            - L1 (absolute sum) norm (vectors only).

        parr
            Parameter name that contains the result.

        normalize
            Normalization key; to be used only for vectors created by ``*VEC``:

            Normalize the vector such that the norm is 1.0. - Do not
            normalize the vector (default).

        Notes
        -----
        The NRM2 option corresponds to the Euclidean or L2 norm and is
        applicable to either vectors or matrices. The NRM1 option corresponds
        to the L1 norm and is applicable to vectors only. The NRMINF option is
        the maximum norm and is applicable to either vectors or matrices.
        """
        command = f"*NRM,{name},{normtype},{parr},{normalize}"
        return self.run(command, **kwargs)

    def remove(self, name="", val1="", val2="", val3="", **kwargs):
        """Suppresses rows or columns of a dense matrix or a vector.

        APDL Command: ``*REMOVE``

        Parameters
        ----------
        name
            Name of the matrix or vector to be revised.

        val1
            First row or column number to suppress if ``name`` is a dense
            matrix.  First value index to suppress if ``name`` is a
            vector.

        Val2
            Last row or column number to suppress if ``name`` is a dense
            matrix.  Last value index to suppress if ``name`` is a
            vector.

        Val3
            Specifies what to remove if ``name`` is a dense matrix.

            * ``"COL"`` : Remove columns of the matrix (default).

            * ``"ROW"`` : Remove rows of the matrix.

        Notes
        -----
        The values of the original matrix or vector specified by Name are
        retained. The matrix or vector is resized to the new number of
        rows and columns.
        """
        return self.run(f"REMOVE,{name},{val1},{val2},{val3}", **kwargs)

    def scal(self, name="", val1="", val2="", **kwargs):
        """Scales a vector or matrix by a constant or a vector.

        APDL Command: ``*SCAL``

        Parameters
        ----------
        name
            Name used to identify the vector or matrix to be scaled. Must
            be specified.

        val1
            When scaling a matrix or a vector by a scalar value, Val1 is
            the real part of the constant to use (default = 1).

            When scaling a matrix or a vector by a vector, Val1 is the
            name of the vector used for the scaling operation.

        val2
            The imaginary part of the constant to use (default = 0).
            This value is used only if the vector or matrix specified by
            Name is complex.

            val2 is only valid for scaling by a constant. It is not
            used when scaling by a vector.

        Notes
        -----
        This command can be applied to vectors and matrices created by the
        ``*VEC``, ``*DMAT`` and ``*SMAT`` commands.

        Data types must be consistent between the vectors and matrices
        being scaled and the scaling vector (or constant value).

        When scaling a matrix with a vector, the matrix must be square
        and the scaling vector must be the same size.

        Scaling a matrix with a vector, is available only on
        MAPDL V23.2 and greater.

        """
        return self.run(f"*SCAL,{name},{val1},{val2}", **kwargs)

    def smat(
        self,
        matrix="",
        type_="",
        method="",
        val1="",
        val2="",
        val3="",
        val4="",
        **kwargs,
    ):
        """Creates a sparse matrix.

        APDL Command: ``*SMAT``

        Parameters
        ----------
        matrix
            Name used to identify the matrix. Must be specified.

        type\\_
            Matrix type:

            Double precision real values (default). - Complex double precision values.

        method
            Method used to create the matrix:

            Copy an existing matrix. - Import the matrix from a file.

        val1, val2, val3, val4
            Additional input. The meaning of Val1 through Val3 will
            vary depending on the specified Method. See in your ansys
            documentation.

        Notes
        -----
        Use the ``*DMAT`` command to create a dense matrix.

        Unlike the ``*DMAT`` command, the ``*SMAT`` command cannot be used to allocate
        a sparse matrix.

        For more information on the NOD2BCS and USR2BCS mapping vectors, see
        Degree of Freedom Ordering in the ANSYS Parametric Design Language
        Guide.

        For more information about .FULL file contents, see the HBMAT in the
        Command Reference.
        """
        command = f"*SMAT,{matrix},{type_},{method},{val1},{val2},{val3},{val4}"
        return self.run(command, **kwargs)

    def starprint(self, matrix="", fname="", **kwargs):
        """Prints the matrix values to a file.

        APDL Command: ``*PRINT``

        Parameters
        ----------
        matrix
            Name of matrix or vector to print. Must be specified.

        fname
            File name. If blank, matrix is written to the output file.

        Notes
        -----
        The matrix may be a dense matrix (``*DMAT``), a sparse matrix
        (``*SMAT``), or a vector (``*VEC``). Only the non-zero entries
        of the matrix are printed.
        """
        command = f"*PRINT,{matrix},{fname}"
        return self.run(command, **kwargs)

    def sort(self, **kwargs):
        """Specifies "Sort settings" as the subsequent status topic.

        APDL Command: SORT

        Notes
        -----
        This is a status [STAT] topic command.  Status topic commands are
        generated by the GUI and will appear in the log file (Jobname.LOG) if
        status is requested for some items under Utility Menu> List> Status.
        This command will be immediately followed by a STAT command, which will
        report the status for the specified topic.

        If entered directly into the program, the STAT command should
        immediately follow this command.
        """
        command = f"SORT,"
        return self.run(command, **kwargs)

    def vec(
        self,
        vector="",
        type_="",
        method="",
        val1="",
        val2="",
        val3="",
        val4="",
        **kwargs,
    ):
        """Creates a vector.

        APDL Command: ``*VEC``

        Parameters
        ----------
        vector
            Name used to identify the vector. Must be specified.

        type_
            Vector type:

            * ``"D"`` : Double precision real values (default).

            * ``"Z"`` : Complex double precision values.

            * ``"I"`` : Integer values.

        method
            Method used to create the vector:

            * ``"ALLOC"`` : Allocate space for a vector (default).

            * ``"RESIZE"`` : Resize an existing vector to a new
              length. Values are kept from the original vector. If the
              length specified by Val1 is greater than the original vector
              length, the additional rows are assigned a value of zero.

            * ``"COPY"`` : Copy an existing vector.

            * ``"IMPORT"`` : Import the vector from a file.

            * ``"LINK"`` : Link to a column of an existing dense ``*DMAT``
              matrix and use it in subsequent vector calculations. Any
              changes to the vector are also made to the corresponding
              matrix column (memory is shared).

            Copy an existing vector. - Import the vector from a file.

        val1, val2, val3, val4, val5
            Additional input. The meaning of ``val1`` through ``val5`` will vary
            depending on the specified Method.  See:
            https://www.mm.bme.hu/~gyebro/files/ans_help_v182/ans_cmd/Hlp_C_VEC.html

        Notes
        -----
        Use the ``*DMAT`` command to create a matrix.

        For more information on the BACK and FORWARD nodal mapping vectors, see
        Degree of Freedom Ordering in the ANSYS Parametric Design Language
        Guide.
        """
        command = f"*VEC,{vector},{type_},{method},{val1},{val2},{val3},{val4}"
        return self.run(command, **kwargs)

    def wrk(self, num="", **kwargs):
        """Sets the active workspace number.

        APDL Command: ``*WRK``

        Parameters
        ----------
        num
            Number of the active memory workspace for APDLMath vector and
            matrices. All the following APDLMath vectors and matrices will
            belong to this memory workspace, until the next call to the ``*WRK``
            command. By default, all the APDLMath objects belong to workspace
            number 1.

        Notes
        -----
        This feature enables you to associate a set of vector and matrices in a
        given memory workspace, so that you can easily manage the free step:

        This feature can be useful to free all the temporary APDLMath variables
        inside a MACRO in one call.
        """
        command = "*WRK,%s" % (str(num))
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

try:
    import click

    _HAS_CLICK = True

except ModuleNotFoundError:
    _HAS_CLICK = False


if _HAS_CLICK:
    ###################################
    # PyMAPDL CLI

    @click.group(invoke_without_command=True)
    @click.pass_context
    def main(ctx):
        pass

    from ansys.mapdl.core.cli.convert import convert
    from ansys.mapdl.core.cli.list_instances import list_instances
    from ansys.mapdl.core.cli.start import start
    from ansys.mapdl.core.cli.stop import stop

    main.add_command(convert)
    main.add_command(start)
    main.add_command(stop)
    main.add_command(list_instances, name="list")


else:

    def main():
        print("PyMAPDL CLI requires 'click' Python package to be installed.")


def old_pymapdl_convert_script_entry_point():
    print(
        """This CLI function has been deprecated. Use the following instead:

pymapdl convert input_file.inp -o output_file.out ...

Go to https://mapdl.docs.pyansys.com/version/dev/user_guide/cli.html for more information.
"""
    )



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from .core import _QueryExecution


class _NodeNormalQueries(_QueryExecution):
    _mapdl = None

    def normnx(self, n1: int, n2: int, n3: int) -> float:
        """X-direction cosine of the normal to the plane containing the given nodes.

        X-direction cosine of the normal to the plane containing nodes
        `n1`, `n2`, and `n3`, reported in the global Cartesian
        coordinate system.

        Parameters
        ----------
        n1 : int
            Node number

        n2 : int
            Node number

        n3 : int
            Node number

        Returns
        -------
        float
            X-direction cosine of the normal

        Examples
        --------
        Here we create three nodes in the y-z plane and interrogate the
        x-component of the normal to that plane, which is trivially 1.0.

        >>> from ansys.mapdl.core import launch_mapdl
        >>> mapdl = launch_mapdl()
        >>> mapdl.prep7()
        >>> n1 = mapdl.n(1, 0, 0, 0)
        >>> n2 = mapdl.n(2, 0, 1, 0)
        >>> n3 = mapdl.n(3, 0, 1, 1)
        >>> q = mapdl.queries
        >>> q.normnx(n1, n2, n3)
        1.0
        """
        return self._run_query(f"NORMNX({n1}, {n2}, {n3})", integer=False)

    def normny(self, n1: int, n2: int, n3: int) -> float:
        """Y-direction cosine of the normal to the plane containing the given nodes.

        Y-direction cosine of the normal to the plane containing nodes
        `n1`, `n2`, and `n3`, reported in the global Cartesian
        coordinate system.

        Parameters
        ----------
        n1 : int
            Node number

        n2 : int
            Node number

        n3 : int
            Node number

        Returns
        -------
        float
            Y-direction cosine of the normal

        Examples
        --------
        Here we create three nodes in the x-z plane and interrogate the
        y-component of the normal to that plane, which is trivially 1.0.

        >>> from ansys.mapdl.core import launch_mapdl
        >>> mapdl = launch_mapdl()
        >>> mapdl.prep7()
        >>> n1 = mapdl.n(1, 0, 0, 0)
        >>> n2 = mapdl.n(2, 1, 0, 0)
        >>> n3 = mapdl.n(3, 1, 0, 1)
        >>> q = mapdl.queries
        >>> q.normny(n1, n2, n3)
        1.0
        """
        return self._run_query(f"NORMNY({n1}, {n2}, {n3})", integer=False)

    def normnz(self, n1: int, n2: int, n3: int) -> float:
        """Z-direction cosine of the normal to the plane containing the given nodes.

        Z-direction cosine of the normal to the plane containing nodes
        `n1`, `n2`, and `n3`, reported in the global Cartesian
        coordinate system.

        Parameters
        ----------
        n1 : int
            Node number

        n2 : int
            Node number

        n3 : int
            Node number

        Returns
        -------
        float
            Z-direction cosine of the normal

        Examples
        --------
        Here we create three nodes in the x-y plane and interrogate the
        z-component of the normal to that plane, which is trivially 1.0.

        >>> from ansys.mapdl.core import launch_mapdl
        >>> mapdl = launch_mapdl()
        >>> mapdl.prep7()
        >>> n1 = mapdl.n(1, 0, 0, 0)
        >>> n2 = mapdl.n(2, 0, 1, 0)
        >>> n3 = mapdl.n(3, 1, 1, 0)
        >>> q = mapdl.queries
        >>> q.normnz(n1, n2, n3)
        1.0
        """
        return self._run_query(f"NORMNZ({n1}, {n2}, {n3})", integer=False)


class _KeypointNormalQueries(_QueryExecution):
    _mapdl = None

    def normkx(self, k1: int, k2: int, k3: int) -> float:
        """X-direction cosine of the normal to the plane containing the given keypoints.

        X-direction cosine of the normal to the plane containing
        keypoints `k1`, `k2`, and `k3`, reported in the global Cartesian
        coordinate system.

        Parameters
        ----------
        k1 : int
            Node number

        k2 : int
            Node number

        k3 : int
            Node number

        Returns
        -------
        float
            X-direction cosine of the normal

        Examples
        --------
        Here we create three keypoints in the y-z plane and interrogate
        the x-component of the normal to that plane, which is trivially
        1.0.

        >>> from ansys.mapdl.core import launch_mapdl
        >>> mapdl = launch_mapdl()
        >>> mapdl.prep7()
        >>> k1 = mapdl.k(1, 0, 0, 0)
        >>> k2 = mapdl.k(2, 0, 1, 0)
        >>> k3 = mapdl.k(3, 0, 1, 1)
        >>> q = mapdl.queries
        >>> q.normnx(k1, k2, k3)
        1.0
        """
        return self._run_query(f"NORMKX({k1}, {k2}, {k3})", integer=False)

    def normky(self, k1: int, k2: int, k3: int) -> float:
        """Y-direction cosine of the normal to the plane containing the given keypoints.

        Y-direction cosine of the normal to the plane containing
        keypoints `k1`, `k2`, and `k3`, reported in the global Cartesian
        coordinate system.

        Parameters
        ----------
        k1 : int
            Node number

        k2 : int
            Node number

        k3 : int
            Node number

        Returns
        -------
        float
            Y-direction cosine of the normal

        Examples
        --------
        Here we create three keypoints in the x-z plane and interrogate
        the y-component of the normal to that plane, which is trivially
        1.0.

        >>> from ansys.mapdl.core import launch_mapdl
        >>> mapdl = launch_mapdl()
        >>> mapdl.prep7()
        >>> k1 = mapdl.k(1, 0, 0, 0)
        >>> k2 = mapdl.k(2, 1, 0, 0)
        >>> k3 = mapdl.k(3, 1, 0, 1)
        >>> q = mapdl.queries
        >>> q.normny(k1, k2, k3)
        1.0
        """
        return self._run_query(f"NORMKY({k1}, {k2}, {k3})", integer=False)

    def normkz(self, k1: int, k2: int, k3: int) -> float:
        """Z-direction cosine of the normal to the plane containing the given keypoints.

        Z-direction cosine of the normal to the plane containing
        keypoints `k1`, `k2`, and `k3`, reported in the global Cartesian
        coordinate system.

        Parameters
        ----------
        k1 : int
            Node number

        k2 : int
            Node number

        k3 : int
            Node number

        Returns
        -------
        float
            Z-direction cosine of the normal

        Examples
        --------
        Here we create three keypoints in the x-y plane and interrogate
        the z-component of the normal to that plane, which is trivially
        1.0.

        >>> from ansys.mapdl.core import launch_mapdl
        >>> mapdl = launch_mapdl()
        >>> mapdl.prep7()
        >>> k1 = mapdl.k(1, 0, 0, 0)
        >>> k2 = mapdl.k(2, 0, 1, 0)
        >>> k3 = mapdl.k(3, 1, 1, 0)
        >>> q = mapdl.queries
        >>> q.normnz(k1, k2, k3)
        1.0
        """
        return self._run_query(f"NORMKZ({k1}, {k2}, {k3})", integer=False)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
These PREP7 commands define the type of elements to be used in the model.
"""
from typing import Optional, Union

from ansys.mapdl.core._commands.parse import parse_et
from ansys.mapdl.core.mapdl_types import MapdlInt


class ElementType:
    def dof(
        self,
        lab1="",
        lab2="",
        lab3="",
        lab4="",
        lab5="",
        lab6="",
        lab7="",
        lab8="",
        lab9="",
        lab10="",
        **kwargs,
    ):
        """Adds degrees of freedom to the current DOF set.

        APDL Command: DOF

        Parameters
        ----------
        lab1, lab2, lab3, . . . , lab10
            Valid labels are: UX, UY, UZ (structural displacements); ROTX,
            ROTY, ROTZ (structural rotations); TEMP, TBOT, TE2, TE3, . . .,
            TTOP (temperatures);  PRES (pressure);  VOLT (voltage); MAG
            (magnetic scalar potential);  AX, AY, AZ (magnetic vector
            potentials);  CURR (current);  EMF (electromotive force drop); CONC
            (concentration); DELETE.

        Notes
        -----
        The degree of freedom (DOF) set for the model is determined from all
        element types defined. This command may be used to add to the current
        set.  The ALL label may be used on some commands to represent all
        labels of the current degree of freedom set for the model.  Issue the
        DOF command with no arguments to list the current set.  Use the DELETE
        label to delete any previously added DOFs and return to the default DOF
        set.
        """
        command = "DOF,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (
            str(lab1),
            str(lab2),
            str(lab3),
            str(lab4),
            str(lab5),
            str(lab6),
            str(lab7),
            str(lab8),
            str(lab9),
            str(lab10),
        )
        return self.run(command, **kwargs)

    def elbow(
        self,
        transkey="",
        tol="",
        dof="",
        cons1="",
        cons2="",
        cons3="",
        cons4="",
        **kwargs,
    ):
        """APDL Command: ELBOW

        Specifies degrees of freedom to be coupled for end release and applies
        section constraints to elbow elements.

        Parameters
        ----------
        transkey
            Pipe-to-elbow transition flag:

            OFF - Do not automatically transition pipes to elbows. (This behavior is the
                  default.)

            ON - Automatically convert straight PIPE289 elements to ELBOW290 elements where it
                 is beneficial. The program converts elements in transition
                 regions where curved ELBOW290 elements are connected to
                 straight PIPE289 elements.

        tol
            Angle tolerance (in degrees) between adjacent ELBOW290 elements.
            The default value is 20. A value of -1 specifies all selected
            ELBOW290 elements.

        dof
            Degrees of freedom to couple:

            ALL - Couple all nodal degrees of freedom (UX, UY, UZ, ROTX, ROTY, and ROTZ). This
                  behavior is the default.

            BALL - Create ball joints (equivalent to releasing ROTX, ROTY, and ROTZ).

        cons1, cons2, cons3, cons4
            Section degrees of freedoms to constrain. If Cons1 through Cons4
            are unspecified, no section constraints are applied:

            SECT  - All section deformation

            SE - Section radial expansion

            SO - Section ovalization

            SW - Section warping

            SRA - Local shell normal rotation about cylindrical axis t2

            SRT - Local shell normal rotation about cylindrical axis t1

        Notes
        -----
        The ELBOW command specifies end releases and section constraints for
        ELBOW290 elements and converts straight PIPE289 elements to ELBOW290
        elements.

        Curved PIPE289 elements are not converted to ELBOW290 elements.

        ELBOW290 elements are generated only if there are existing ELBOW290
        elements in the curved areas.

        The command works on currently selected nodes and elements. It creates
        end releases on any two connected elbow elements whose angle at
        connection exceeds the specified tolerance. From within the GUI, the
        Picked node option generates an end release and section constraints at
        the selected node regardless of the angle of connection (that is, the
        angle tolerance [TOL ] is set to -1).

        Elbow and pipe elements must share the same section ID in order for the
        pipe-to-elbow transition to occur.

        To list the elements altered by the ELBOW command, issue an ELIST
        command.

        To list the coupled sets generated by the ELBOW command, issue a CPLIST
        command.

        To list the section constraints generated by the ELBOW command, issue a
        DLIST command.
        """
        command = "ELBOW,%s,%s,%s,%s,%s,%s,%s" % (
            str(transkey),
            str(tol),
            str(dof),
            str(cons1),
            str(cons2),
            str(cons3),
            str(cons4),
        )
        return self.run(command, **kwargs)

    def et(
        self,
        itype: MapdlInt = "",
        ename: Union[str, int] = "",
        kop1: MapdlInt = "",
        kop2: MapdlInt = "",
        kop3: MapdlInt = "",
        kop4: MapdlInt = "",
        kop5: MapdlInt = "",
        kop6: MapdlInt = "",
        inopr: MapdlInt = "",
        **kwargs,
    ) -> Optional[int]:
        """Define a local element type from the element library.

        APDL Command: ET

        Parameters
        ----------
        itype
            Arbitrary local element type number. Defaults to 1 +
            current maximum.

        ename
            Element name (or number) as given in the element library
            in Chapter 4 of the Element Reference. The name consists
            of a category prefix and a unique number, such as PIPE288.
            The category prefix of the name (PIPE for the example) may
            be omitted but is displayed upon output for clarity. If
            ``ename=0``, the element is defined as a null element.

        kop1, kop2, kop3, kop4, kop5, kop6
            KEYOPT values (1 through 6) for this element, as described
            in the Element Reference.

        inopr
            If 1, suppress all element solution printout for this
            element type.

        Notes
        -----
        The ET command selects an element type from the element
        library and establishes it as a local element type for the
        current model.  Information derived from the element type is
        used for subsequent commands, so the ET command(s) should be
        issued early. (The Element Reference describes the available
        elements.)

        A special option, ``ename=0``, permits the specified element
        type to be ignored during solution without actually removing
        the element from the model. Ename may be set to zero only
        after the element type has been previously defined with a
        nonzero Ename.  The preferred method of ignoring elements is
        to use the select commands (such as ESEL).

        KOPn are element option keys. These keys (referred to as
        KEYOPT(n)) are used to turn on certain element options for
        this element. These options are listed under "KEYOPT" in the
        input table for each element type in the Element Reference.
        KEYOPT values include stiffness formulation options, printout
        controls, and various other element options. If KEYOPT(7) or
        greater is needed, input their values with the KEYOPT command.

        The ET command only defines an element type local to your
        model (from the types in the element library). The TYPE or
        similar [KATT, LATT, AATT, or VATT] command must be used to
        point to the desired local element type before meshing.

        To activate the ANSYS program's LS-DYNA explicit dynamic
        analysis capability, use the ET command
        to choose an element that works only with LS-DYNA (such as
        SHELL163).

        Examples
        --------
        Define an element type.  Allow MAPDL to pick your the element
        type.

        >>> etype_num = mapdl.et('', 'SURF154')
        >>> etype_num
        1

        Define an element type while specifying the element type
        number.

        >>> etype_num = mapdl.et(2, 'SOLID186')
        >>> etype_num
        2

        """
        command = (
            f"ET,{itype},{ename},{kop1},{kop2},{kop3},{kop4}," f"{kop5},{kop6},{inopr}"
        )
        return parse_et(self.run(command, **kwargs))

    def etchg(self, cnv: str = "", **kwargs) -> Optional[str]:
        """Changes element types to their corresponding types.

        APDL Command: ETCHG

        Parameters
        ----------
        cnv
            Converts the element types to the corresponding type.
            Valid labels
            are:

            ETI - Explicit to Implicit

            ITE - Implicit to Explicit

            TTE - Thermal to Explicit

            TTS - Thermal to Structural

            STT - Structural to Thermal

            MTT - Magnetic to Thermal

            FTS - Fluid to Structural

            ETS - Electrostatic to Structural

            ETT - Electrical to Thermal

        Notes
        -----
        Changes the currently defined element types to their
        corresponding types.  Elements without a companion element
        (listed above) are not switched and should be switched with
        the ET command to an appropriate element type or to a null
        element. The KEYOPT values for the switched element types are
        reset to zero or to their default values. You must
        check these values to see if they are still meaningful.
        Additionally, if Cnv = ETI, ITE, or TTE, all real constants
        are set to zero.

        If Cnv = ITE, you will need to choose a material model that
        corresponds to your previously-defined material properties.
        If working interactively, you will be prompted to do so.
        """
        command = f"ETCHG,{cnv}"
        return self.run(command, **kwargs)

    def etcontrol(
        self, eltech: str = "", eldegene: str = "", **kwargs
    ) -> Optional[str]:
        """Control the element technologies used in element
        formulation (for applicable elements).

        APDL Command: ETCONTROL

        Parameters
        ----------
        eltech
            Element technology control.  One of the following:

            - ``"SUGGESTION"`` : The program offers a suggestion for the
              best element technology before solving.  If necessary,
              mixed u-P (KEYOPT(6)) is also included and reset. This
              behavior is the default.

            - ``"SET"`` : The program informs you of the best settings
              and resets any applicable KEYOPT settings
              automatically. This action overrides any previous manual
              settings.

            - ``"OFF"`` : Deactivates automatic selection of element
              technology. No suggestions are issued, and no
              automatic resetting occurs.

        eldegene
            Element degenerated shape control.  One of the following:

            - ``"ON"`` - If element shapes are degenerated, the
              degenerated shape function is employed and enhanced
              strain, simplified enhanced strain, and B-bar formulations
              are turned off (default).

            - ``"OFF"`` - If element shapes are degenerated, regular
              shape functions are still used, and the specified element
              technologies (e.g., enhanced strain, B-bar, uniform
              reduced integration) are still used.

        Notes
        -----
        The command default is ``mapdl.etcontrol('SUGGESTION', 'ON')``

        This command is valid for elements SHELL181, PLANE182,
        PLANE183, SOLID185, SOLID186, SOLID187, BEAM188, BEAM189,
        SHELL208, SHELL209, PLANE223, SOLID226, SOLID227, REINF264,
        SOLID272, SOLID273, SHELL281, SOLID285, PIPE288, PIPE289,
        ELBOW290.

        For more information, see Automatic Selection of Element
        Technologies and Formulations in the Element Reference.

        Examples
        --------
        Enable element tech control and degenerated shape control.

        >>> mapdl.et(1, 'SOLID186')
        >>> output = mapdl.etcontrol(eltech='SET', eldegene='ON')
        >>> print(output)
        ELEMENT TECHNOLOGY CONTROL PARAMETER FOR APPLICABLE ELEMENTS = SET.
         DEGENERATED ELEMENT SHAPE CONTROL PARAMETER FOR APPLICABLE ELEMENTS = ON.

        """
        return self.run(f"ETCONTROL,{eltech},{eldegene}", **kwargs)

    def etdele(
        self,
        ityp1: Union[str, int] = "",
        ityp2: MapdlInt = "",
        inc: MapdlInt = "",
        **kwargs,
    ) -> Optional[str]:
        """APDL Command: ETDELE

        Deletes element types.

        Parameters
        ----------
        ityp1, ityp2, inc
            Deletes element types from ``ityp1`` to ``ityp2``
            (defaults to ``ityp1``) in steps of ``inc`` (defaults to
            1). If ``ityp1='ALL'``, ``ityp2`` and ``inc`` are ignored
            and all element types are deleted.  Element types are
            defined with the ``et`` command.

        Examples
        --------
        Create and delete an element type.

        >>> mapdl.et(1, 'SOLID186')
        >>> mapdl.etdele(1)

        """
        command = f"ETDELE,{ityp1},{ityp2},{inc}"
        return self.run(command, **kwargs)

    def etlist(
        self,
        ityp1: MapdlInt = "",
        ityp2: MapdlInt = "",
        inc: MapdlInt = "",
        **kwargs,
    ) -> Optional[str]:
        """Lists currently defined element types.

        APDL Command: ETLIST

        Parameters
        ----------
        ityp1, ityp2, inc
            Lists element types from ITYP1 to ITYP2 (defaults to
            ITYP1) in steps of INC (defaults to 1). If ITYP1 = ALL (
            default), ITYP2 and INC are ignored and all element types
            are listed.

        Notes
        -----
        This command is valid in any processor.
        """
        command = f"ETLIST,{ityp1},{ityp2},{inc}"
        return self.run(command, **kwargs)

    def keyopt(self, itype="", knum="", value="", **kwargs):
        """APDL Command: KEYOPT

        Sets element key options.

        Parameters
        ----------
        itype
            Element type number as defined on the ET command. The label GCN is
            also valid input for general contact elements (see Notes).

        knum
            Number of the KEYOPT to be defined (KEYOPT(KNUM)).

        value
            Value of this KEYOPT.

        Notes
        -----
        Alternative to inputting KEYOPT values on ET command.  Must be used if
        KEYOPT(7) or greater values are to be input. ITYPE must first be
        defined with the ET command.

        Specify ITYPE = GCN to set element key options for all contact elements
        types used in any existing general contact definitions (that is,
        contact elements having a real constant set number = 0).
        """
        command = "KEYOPT,%s,%s,%s" % (str(itype), str(knum), str(value))
        return self.run(command, **kwargs)

    def nsvr(self, itype="", nstv="", **kwargs):
        """APDL Command: NSVR

        Defines the number of variables for user-programmable element options.

        Parameters
        ----------
        itype
            Element type number as defined on the ET command.

        nstv
            Number of extra state variables to save (must be no more than 840).

        Notes
        -----
        Defines the number of extra variables that need to be saved for user-
        programmable (system-dependent) element options, e.g., material laws
        through user subroutine USERPL.  ITYPE must first be defined with the
        ET command.
        """
        command = "NSVR,%s,%s" % (str(itype), str(nstv))
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class LoadStepOperations:
    def lsclear(self, lab="", **kwargs):
        """Clears loads and load step options from the database.

        APDL Command: LSCLEAR

        Parameters
        ----------
        lab
            Label identifying the data to be cleared:

            SOLID - Delete only solid model loads.

            FE - Delete only finite element loads.

            INER - Delete only inertia loads (ACEL, etc.).

            LFACT - Initialize only load factors (on DCUM, FCUM, SFCUM, etc.).

            LSOPT - Initialize only load step options.

            ALL - Delete all loads and initialize all load step options and load factors.

        Notes
        -----
        Loads are deleted, and load step options are initialized to their
        default values.

        This command is also valid in PREP7.
        """
        command = f"LSCLEAR,{lab}"
        return self.run(command, **kwargs)

    def lsdele(self, lsmin="", lsmax="", lsinc="", **kwargs):
        """Deletes load step files.

        APDL Command: LSDELE

        Parameters
        ----------
        lsmin, lsmax, lsinc
            Range of load step files to be deleted, from LSMIN to LSMAX in
            steps of LSINC.  LSMAX defaults to LSMIN, and LSINC defaults to 1.
            If LSMIN = ALL, all load step files are deleted (and LSMAX and
            LSINC are ignored).  The load step files are assumed to be named
            Jobname.Sn, where n is a number assigned by the LSWRITE command (01
            --09,10,11, etc.).  On systems with a 3-character limit on the
            extension, the "S" is dropped for numbers > 99.

        Notes
        -----
        Deletes load step files in the current directory (written by the
        LSWRITE command).

        This command is also valid in PREP7.
        """
        command = f"LSDELE,{lsmin},{lsmax},{lsinc}"
        return self.run(command, **kwargs)

    def lsread(self, lsnum="", **kwargs):
        """Reads load and load step option data into the database.

        .. warning:: This command can only run in non-interactive mode.
            Please visit `Unsupported "Interactive" Commands
            <https://mapdl.docs.pyansys.com/version/stable/user_guide/mapdl.html#unsupported-interactive-commands>`_
            for further information.

        APDL Command: LSREAD

        Parameters
        ----------
        lsnum
            Identification number of the load step file to be read.  Defaults
            to 1 + highest number read in the current session.  Issue
            LSREAD,STAT to list the current value of LSNUM.  Issue LSREAD,INIT
            to reset LSNUM to 1.  The load step files are assumed to be named
            Jobname.Sn, where n is a number assigned by the LSWRITE command (01
            --09,10,11, etc.).  On systems with a 3-character limit on the
            extension, the "S" is dropped for LSNUM > 99.

        Notes
        -----
        Reads load and load step option data from the load step file into the
        database.  LSREAD will not clear the database of all current loads.
        However, if a load is respecified with LSREAD, then it will overwrite
        the existing load. See the LSWRITE command to write load step files,
        and the LSDELE command to delete load step files.  LSREAD removes any
        existing SFGRAD specification.

        This command is also valid in PREP7.

        Examples
        --------
        Demonstrate writing out load steps using :func:`lswrite
        <ansys.mapdl.core.Mapdl.lswrite> and reading them back in using
        ``lsread``.

        >>> from ansys.mapdl.core import launch_mapdl
        >>> mapdl = launch_mapdl()
        >>> mapdl.clear()
        >>> mapdl.prep7()

        Build a 5 x 5 flat plate out of shell181 elements.

        >>> mapdl.rectng(0, 5, 0, 5)
        >>> mapdl.et(1, 'SHELL181')
        >>> mapdl.mp('ex', 1, 10.0e5)
        >>> mapdl.sectype(1, 'shell')
        >>> mapdl.secdata(0.1)  #  0.1 thick
        >>> mapdl.esize(1)
        >>> mapdl.amesh('all')

        Fix the four corners

        >>> mapdl.d('node(0,0,0)', 'all')
        >>> mapdl.d('node(0,5,0)', 'all')
        >>> mapdl.d('node(5,5,0)', 'all')
        >>> mapdl.d('node(5,0,0)', 'all')

        Enter the solution routine and define a force at (2,2,0).

        >>> mapdl.slashsolu()
        >>> mapdl.antype('static')
        >>> mapdl.f('node(2,2,0)', 'fz', -10)

        Write load out as load step 1 and delete all loads and displacements.

        >>> mapdl.lswrite(1)
        >>> mapdl.fdele('all', 'all')
        >>> mapdl.ddele('all', 'all')

        Read back in the loads and list them.

        >>> mapdl.lsread(1)
        >>> mapdl.flist()
        LIST NODAL FORCES FOR SELECTED NODES         1 TO       36 BY        1
        CURRENTLY SELECTED NODAL LOAD SET= FX   FY   FZ   MX   MY   MZ

        *** MAPDL - ENGINEERING ANALYSIS SYSTEM  RELEASE 2022 R2          22.2     ***
        Ansys Mechanical Enterprise
        00000000  VERSION=LINUX x64     10:15:14  AUG 22, 2022 CP=      0.561

            NODE  LABEL     REAL           IMAG
               26  FZ     -10.0000000      0.00000000


        """
        command = f"LSREAD,{lsnum}"
        return self.run(command, **kwargs)

    def lssolve(self, lsmin="", lsmax="", lsinc="", **kwargs):
        """Reads and solves multiple load steps.

        APDL Command: LSSOLVE

        Parameters
        ----------
        lsmin, lsmax, lsinc
            Range of load step files to be read and solved, from
            ``lsmin`` to ``lsmax`` in steps of ``lsinc``.  ``lsmax``
            defaults to ``lsmin``, and ``lsinc`` defaults to 1. If
            ``lsmin`` is blank, a brief command description is
            displayed.  The load step files are assumed to be named
            Jobname.Sn, where n is a number assigned by the
            ``lswrite`` command (01--09, 10, 11, etc.).  On systems
            with a 3-character limit on the extension, the "S" is
            dropped for numbers > 99.

        Notes
        -----
        ``lssolve`` invokes an ANSYS macro to read and solve multiple
        load steps.  The macro loops through a series of load step
        files written by the LSWRITE command.  The macro file called
        by ``lssolve`` is called LSSOLVE.MAC.

        ``lssolve`` cannot be used with the birth-death option.

        ``lssolve`` is not supported for cyclic symmetry analyses.

        ``lssolve`` does not support restarts.

        Examples
        --------
        Write the load and load step option data to a file and solve
        it.  In this case, write the second load step.

        >>> mapdl.lswrite(2)
        >>> mapdl.lssolve(1, 2)

        """
        return self.run(f"LSSOLVE,{lsmin},{lsmax},{lsinc}", **kwargs)

    def lswrite(self, lsnum="", **kwargs):
        """Writes load and load step option data to a file.

        APDL Command: LSWRITE

        Parameters
        ----------
        lsnum
            Number to be assigned to the load step file name for identification
            purposes.  Defaults to 1 + highest LSNUM used in the current
            session.  Issue LSWRITE,STAT to list the current value of LSNUM.
            Issue LSWRITE,INIT to reset to 1.  The load step file will be named
            Jobname.Sn, where n is the specified LSNUM value (preceded by "0"
            for values 1-9).  On systems with a 3-character limit on the file
            name extension, the "S" is dropped for LSNUM > 99.

        Notes
        -----
        Writes all load and load step option data for the selected model to a
        load step file for later use.  LSWRITE does not capture changes made to
        real constants (R), material properties (MP), couplings (CP), or
        constraint equations (CE).

        Solid model loads will not be saved if the model is not meshed. Solid
        model loads, if any, are transferred to the finite element model. Issue
        LSCLEAR,FE to delete finite element loads.

        One file is written for each load step. Use the LSREAD command to read
        a single load step file, and the LSDELE command to delete load step
        files.  Use the LSSOLVE command to read and solve the load steps
        sequentially.

        Solution control commands are typically not written to the file unless
        you specifically change a default solution setting.

        LSWRITE does not support the following commands: DJ, FJ, GSBDATA,
        GSGDATA, ESTIF, EKILL, EALIVE, MPCHG, and OUTRES. These commands will
        not be written to the load step file.

        LSWRITE cannot be used with the birth-death option.

        This command is also valid in PREP7.
        """
        command = f"LSWRITE,{lsnum}"
        return self.run(command, **kwargs)
